<?php
if (!defined('ABSPATH')) {
    exit();
}

if (!class_exists('App_Expert_Page_Manager')) {

    class App_Expert_Page_Manager
    {

        const PAGE_RENDERER_SLUG = 'app-experts-page-renderer';

        const PAGE_TEMPLATE_COCOBASIC_THEME = 'cocobasic-shortcode';
        const PAGE_TEMPLATE_MEELO_WP_THEME = 'meelo-wp';
        const PAGE_TEMPLATE_ELEMENTOR_THEME = 'elementor';

        const PAGE_MOBILE_PREVIEW_QUERY_VAR = 'appexperts_mobile_preview';


        public function __construct()
        {

            // Custom page preview for webview
            add_action('pre_get_posts', array($this, 'change_wp_page_alter_query'));


            // Render custom checkout page
            add_filter('template_include', array($this, 'render_webview_page'), 12, 1); // After woocoomerce and elementor

            // Add custom style
            add_action('wp_enqueue_scripts', [$this, 'enqueue_custom_page_style']);
        }


        function change_wp_page_alter_query($query)
        {

            if (!$this->is_plugin_page_renderer_page() ||  !$query->is_main_query()) {
                return;
            }

            // Get the other page to display, will be sent in url in 'display_page_id' parameter
            $page_id = $_GET['display_page_id'];
            if (!$page_id) {
                return;
            }

            // add WPML support
            // if (in_array('sitepress-multilingual-cms/sitepress.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            //     if (isset($_GET['appexpertlang']) && $_GET['appexpertlang']) {
            //         $page_id = apply_filters('wpml_object_id', $page_id, 'page', true, $_GET['appexpertlang']);
            //     }
            // }

            $query->set('post_type', 'page');
            $query->set('page_id', $page_id);
        }


        public function enqueue_custom_page_style()
        {
            if (!$this->is_plugin_page_renderer_page() && !$this->is_page_for_mobile_preview()) {
                return;
            }

            wp_enqueue_style('app-expert-menu-style', APP_EXPERT_URL . 'assets/css/menu-styles.css', false, '1.0', 'all');
        }


        public function render_webview_page($template)
        {

            if ($this->is_plugin_page_renderer_page()) {

                if ((false !== strpos($template, self::PAGE_TEMPLATE_COCOBASIC_THEME))
                    || (false !== strpos($template, self::PAGE_TEMPLATE_ELEMENTOR_THEME))
                    || (false !== strpos($template, self::PAGE_TEMPLATE_MEELO_WP_THEME))
                ) {


                    $permalink = get_the_permalink();
                    $lang = "en";

                    // Apply localization if exists
                    if (in_array('sitepress-multilingual-cms/sitepress.php', apply_filters('active_plugins', get_option('active_plugins')))) {
                        $lang = (isset($_GET['display_language']) && $_GET['display_language']) ? $_GET['display_language'] : null;
                        $permalink = apply_filters('wpml_permalink', $permalink,  $lang);
                    }
                    $lang = $lang ? $lang : "en";

                    $permalink = add_query_arg([
                        'lang' => $lang,
                        self::PAGE_MOBILE_PREVIEW_QUERY_VAR => '1',
                    ], $permalink);

                    wp_redirect($permalink);
                    exit;
                }


                return APP_EXPERT_PATH . "templates/cleanpage.php";
            }

            return $template;
        }



        /**
         * Check if the current page visited is the plugin page renderer page
         * 
         * @return bool
         */
        protected function is_plugin_page_renderer_page()
        {
            $appexpert_rendere_page = @get_page_by_path(self::PAGE_RENDERER_SLUG);
            if (!$appexpert_rendere_page || is_admin()) {
                return false;
            }

            $request_uri = $_SERVER['REQUEST_URI'];
            $relative_page_permalink = str_replace(home_url(), '', get_permalink($appexpert_rendere_page->ID));
            return (false !== strpos($request_uri, $relative_page_permalink));
        }

        /**
         * Check if the current page visited is from our page renderer
         * 
         * @return bool
         */
        protected function is_page_for_mobile_preview()
        {
            return isset($_GET[self::PAGE_MOBILE_PREVIEW_QUERY_VAR]) && !empty($_GET[self::PAGE_MOBILE_PREVIEW_QUERY_VAR]);
        }
    }
}
new App_Expert_Page_Manager();
