<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Authentication')){

	class App_Expert_Authentication {


		/**
		 * @var string
		 */
		private $user_id;

		/**
		 * @var WP_Error
		 */
		private $app_expert_authentication_error;

		public function __construct() {

			add_filter('determine_current_user' , array($this , 'authenticate_api_urls' ) , 10);
			add_filter('rest_pre_dispatch' , array($this , 'authentication_error_pre_dispatch' ) , 10 , 3);

		}



		public function authenticate_api_urls($user_id){

			$request_uri = $_SERVER['REQUEST_URI'];
			if ($user_id ||
			    $this->is_method_GET($request_uri)||
			    $this->is_login_path($request_uri) ||
			    $this->is_register_path($request_uri)||
			    $this->is_reset_password_path($request_uri)||
			    $this->is_refresh_token_path($request_uri) ||
				!$this->is_api_request_url($request_uri) ){

				return $user_id;
			}

			$authorization_header = $_SERVER['HTTP_AUTHORIZATION'];
			$token_auth_result = self::authenticate_user_token($authorization_header);

			if (!is_wp_error($token_auth_result)){
				//append user id to request so that u dont need to decode the token again to get id
				$this->user_id = $token_auth_result;
				$user_id = $this->user_id;
			}else{
				$this->app_expert_authentication_error = $token_auth_result;
			}

			return $user_id;
		}

		/**
		 * Authenticate user token
		 *
		 * @param  string $authorization_header
		 *
		 * @return int|WP_Error int user id or WP_Error wordpress error
		 */
		public static function authenticate_user_token($authorization_header){
			
			
			if (!self::is_authorization_header_present($authorization_header)){
				return new WP_Error(
					'app_expert_no_authorization_header',
					__('Authorization Header must be provided', 'app-expert'),
					array(
						'status' => 401,
					)
				);
			}

			if (!self::is_authorization_header_valid($authorization_header)){
				return new WP_Error(
					'app_expert_authorization_header_invalid',
					__('Authorization Header Format is incorrect', 'app-expert'),
					array(
						'status' => 401,
					)
				);
			}

			$token = explode(' ' , $authorization_header)[1];

			$jwt_helper = new App_Expert_JWT();
			$decode_result = $jwt_helper->decode_token($token);

			if (is_a($decode_result , 'Exception')){
				return new WP_Error(
					$decode_result->getMessage(),
					__('Something happened while decoding the token', 'app-expert'),
					array(
						'status' => 401,
					)
				);
			}

			return $jwt_helper->get_user_id($decode_result);

		}

		/**
		 * @param string $request_uri
		 *
		 * @return bool
		 */
		private function is_method_GET(string  $request_uri){
			return $_SERVER['REQUEST_METHOD'] === 'GET' &&
			       strpos($request_uri , APP_EXPERT_API_NAMESPACE.'/users/me') === false;
		}

		/**
		 * @param string $request_uri
		 *
		 * @return false|int
		 */
		private function is_login_path(string  $request_uri){
			return strpos($request_uri , APP_EXPERT_API_NAMESPACE.App_Expert_Login::LOGIN_PATH) > 0;
		}

		/**
		 * @param string $request_uri
		 *
		 * @return false|int
		 */
		private function is_register_path(string  $request_uri){
			return strpos($request_uri , APP_EXPERT_API_NAMESPACE.'/register') > 0;
		}


		/**
		 * @param string $request_uri
		 *
		 * @return false|int
		 */
		private function is_reset_password_path(string  $request_uri){
			return strpos($request_uri , APP_EXPERT_API_NAMESPACE.'/users/forgot_password') > 0;
		}

		/**
		 * @param string $request_uri
		 *
		 * @return bool
		 */
		private function is_refresh_token_path(string  $request_uri){
			return strpos($request_uri , APP_EXPERT_API_NAMESPACE.App_Expert_Login::REFRESH_TOKEN_PATH) > 0;

		}

		/**
		 * @param string $request_uri
		 *
		 * @return bool
		 */
		private function is_api_request_url(string  $request_uri){
			$valid_api_uri = strpos($request_uri, APP_EXPERT_API_NAMESPACE);

			return $valid_api_uri !== false;
		}


		/**
		 * @param $authorization_header
		 *
		 * @return bool
		 */
		public static function is_authorization_header_present($authorization_header){
			return isset($authorization_header) &&
			       !empty($authorization_header);
		}

		/**
		 * @param string $authorization_header
		 *
		 * @return bool
		 */
		public static function is_authorization_header_valid(string  $authorization_header){

			$bearer_token = explode(' ' , $authorization_header);

			return count($bearer_token) === 2 && $bearer_token[0] === 'Bearer';

		}

		public function authentication_error_pre_dispatch( $result , WP_REST_Server $server,  WP_REST_Request $request){
			if (is_wp_error($this->app_expert_authentication_error)){
				return $this->app_expert_authentication_error;
			}

			if ($this->is_user_id_set())
				$request->set_param(App_Expert_Request::REQUEST_PARAMETER_NAME , array(
					'id' => $this->user_id
				));
			return $result;

		}

		/**
		 * @return bool
		 */
		private function is_user_id_set(){
			return isset($this->user_id) && !empty($this->user_id);
		}

	}
}
new App_Expert_Authentication();