<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_CPT_Controller')){

	class App_Expert_CPT_Controller extends WP_REST_Posts_Controller {


		/**
		 * @var App_Expert_Response_Meta_Data
		 */
		private $app_expert_response_meta_data;


		public function __construct($post_type){

			parent::__construct($post_type);
			$this->namespace = APP_EXPERT_API_NAMESPACE;
			$this->app_expert_response_meta_data= new App_Expert_Response_Meta_Data($this->get_response_meta_data());

		}

		public function register_routes() {

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base,
				array(
					array(
						'methods'             => WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_items' ),
						'args'                => $this->get_collection_params(),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);

			$schema        = $this->get_item_schema();
			$get_item_args = array(
				'context' => $this->get_context_param( array( 'default' => 'view' ) ),
			);
			if ( isset( $schema['properties']['password'] ) ) {
				$get_item_args['password'] = array(
					'description' => __( 'The password for the post if it is password protected.' ),
					'type'        => 'string',
				);
			}
			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base . '/(?P<id>[\d]+)',
				array(
					'args'   => array(
						'id' => array(
							'description' => __( 'Unique identifier for the object.' ),
							'type'        => 'integer',
						),
					),
					array(
						'methods'             => WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_item' ),
						'args'                => $get_item_args,
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);
		}


		public function get_items($request){
			$response =  parent::get_items($request);
			if (is_wp_error($response))
				return $response;

			$response = $this->app_expert_response_meta_data->attach_meta_data($response , false);
			return App_Expert_Response::get_api_list_success_response(
				$response,
				'app_expert_posts_retrieved',
				'retrieved all posts'
			);
		}

		public function get_item($request){
			$response =  parent::get_item($request);
			if (is_wp_error($response))
				return $response;

			$response = $this->app_expert_response_meta_data->attach_meta_data($response , true);

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_post_retrieved',
				'retrieved requested post'
			);
		}

		/**
		 * @return array
		 */
		private function get_response_meta_data(){
			return array(
				new App_Expert_Image_Meta_Data(),
				new App_Expert_Author_Meta_Data(),
				new App_Expert_CPT_Term_Meta_Data()
			);
		}

	}


}