<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_CT_Controller')){

	class App_Expert_CT_Controller extends WP_REST_Terms_Controller {

		/**
		 * @var App_Expert_Hierarchy_Handler
		 */
		private $app_expert_hierarchy_handler;

		public function __construct($taxonomy) {
			parent::__construct($taxonomy);
			$this->namespace = APP_EXPERT_API_NAMESPACE;

			$this->app_expert_hierarchy_handler = new App_Expert_Hierarchy_Handler();

		}

		public function register_routes() {

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base,
				array(
					array(
						'methods'             => WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_items' ),
						'args'                => $this->get_collection_params(),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base . '/(?P<id>[\d]+)',
				array(
					'args'   => array(
						'id' => array(
							'description' => __( 'Unique identifier for the term.' ),
							'type'        => 'integer',
						),
					),
					array(
						'methods'             => WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_item' ),
						'args'                => array(
							'context' => $this->get_context_param( array( 'default' => 'view' ) ),
						),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);
		}

		public function get_items($request){
			$response = $this->app_expert_hierarchy_handler->get_taxonomy_children_response($this->taxonomy , 0 , $request , $this);
			$response = rest_ensure_response( $response );

			if (is_wp_error($response))
				return $response;

			return App_Expert_Response::get_api_list_success_response(
				$response,
				'app_expert_categories_retrieved',
				'retrieved all categories'
			);
		}

		public function get_item($request){
			$taxonomy = get_term($request['id'] , $this->taxonomy);

			if (is_wp_error($taxonomy))
				return $taxonomy;

			$response = $this->app_expert_hierarchy_handler->get_term_prepared($this->taxonomy , $taxonomy , $request , array() , $this);
			$response = rest_ensure_response( $response );

			if (is_wp_error($response))
				return $response;

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_category_retrieved',
				'retrieved requested category'
			);
		}





	}
}

