<?php
if (!defined('ABSPATH')) {
	exit();
}

if (!class_exists('App_Expert_Register_Controller')) {

	class App_Expert_Register_Controller extends  WP_REST_Users_Controller
	{

		/**
		 * @var App_Expert_Login
		 */
		private  $app_Expert_login;

		public function __construct()
		{
			parent::__construct();
			$this->rest_base = '/register';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
			$this->app_Expert_login = new App_Expert_Login();
		}

		public function register_routes()
		{
			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base,
				array(
					array(
						'methods'  => WP_REST_Server::CREATABLE,
						'callback' => array($this, 'create_item'),
						'args'     => $this->get_endpoint_args_for_item_schema(WP_REST_Server::CREATABLE),
					),
					'schema' => array($this, 'get_public_item_schema'),
				)
			);
		}

		public function create_item($request)
		{
			$request = App_Expert_Request::remove_user_meta_data_register($request);

			if (!$this->is_username_exists($request)) {
				$username = $this->get_generated_username($request);
				$request->set_param('username', $username);
			}

			$response =  parent::create_item($request);


			if (is_wp_error($response))
				return $this->handle_error_response($response);

			$response = $this->app_Expert_login->handle_user_login($request);

			if (is_wp_error($response))
				return $response;

			$response['code'] = 'app_expert_created_user';
			$response['message'] = 'User Created';

			return $response;
		}

		/**
		 * @param WP_REST_Request $request
		 * @param string $suffix
		 *
		 * @return string
		 */
		private function get_generated_username(WP_REST_Request $request, $suffix = '')
		{
			$email = $request->get_param('email');

			$email_parts = explode('@', $email);

			$email_username = $email_parts[0];

			// Exclude common prefixes.
			if (in_array(
				$email_username,
				array(
					'sales',
					'hello',
					'mail',
					'contact',
					'info',
				),
				true
			)) {
				// Get the domain part.
				$email_username = $email_parts[1];
			}

			$username_parts[] = sanitize_user($email_username, true);

			$username = strtolower(implode('.', $username_parts));

			if ($suffix) {
				$username .= $suffix;
			}

			$illegal_logins = (array) apply_filters('illegal_user_logins', array());

			if (in_array(strtolower($username), array_map('strtolower', $illegal_logins), true)) {

				return $this->get_generated_username($request, '-' . zeroise(wp_rand(0, 9999), 4));
			}

			if (username_exists($username)) {
				$suffix = '-' . zeroise(wp_rand(0, 9999), 4);
				return $this->get_generated_username($request, $suffix);
			}

			return $username;
		}


		/**
		 * Sepcial handling for existing email response to fix its error code
		 *
		 * @param WP_Error $response
		 *
		 * @return WP_Error
		 */
		private function handle_error_response(WP_Error $response)
		{
			if (
				property_exists($response, 'errors')
				&& isset($response->errors['existing_user_email'])
			) {
				return new WP_Error(
					'existing_user_email',
					__('This email is already used.', 'app-expert'),
					array(
						'status' => 400,
						'params' => array(
							'email' => __('This email is already used.', 'app-expert')
						)
					)
				);
			}

			return $response;
		}


		/**
		 * @param string $username
		 *
		 * @return string|WP_Error
		 */
		private function sanitize_username(string $username)
		{

			if (!validate_username($username)) {
				return new WP_Error('rest_user_invalid_username', __('Username contains invalid characters.', 'app-expert'), array('status' => 400));
			}

			/** This filter is documented in wp-includes/user.php */
			$illegal_logins = (array) apply_filters('illegal_user_logins', array());

			if (in_array(strtolower($username), array_map('strtolower', $illegal_logins))) {
				return new WP_Error('rest_user_invalid_username', __('Sorry, that username is not allowed.', 'app-expert'), array('status' => 400));
			}

			return $username;
		}
		/**
		 * @return array
		 */
		public function get_item_schema()
		{

			return array(
				'$schema'    => 'http://json-schema.org/draft-04/schema#',
				'title'      => 'user',
				'type'       => 'object',
				'properties' => array(
					'username'           => array(
						'description' => __('Login name for the user.'),
						'type'        => 'string',
						'context'     => array('edit'),
						'arg_options' => array(
							'sanitize_callback' => array($this, 'check_username'),
						),
					),
					'first_name'         => array(
						'description' => __('First name for the user.'),
						'type'        => 'string',
						'context'     => array('edit'),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
						'required' => true
					),
					'last_name'          => array(
						'description' => __('Last name for the user.'),
						'type'        => 'string',
						'context'     => array('edit'),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
						'required' => true
					),
					'email'              => array(
						'description' => __('The email address for the user.'),
						'type'        => 'string',
						'format'      => 'email',
						'context'     => array('edit'),
						'required'    => true,
					),
					'password'           => array(
						'description' => __('Password for the user (never included).'),
						'type'        => 'string',
						'context'     => array(), // Password is never displayed.
						'required'    => true,
						'arg_options' => array(
							'sanitize_callback' => array($this, 'check_user_password'),
						),
					),
					'password_confirmation' => array(
						'description' => __('Password Confirmation  for the user (never included).'),
						'type'        => 'string',
						'context'     => array(),
						'required'    => true,
						'arg_options' => array(
							'sanitize_callback' => array($this, 'check_confirm_password'),
						),
					)
				),
			);
		}


		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_user_password($value, $request, $param){
			$password = (string) $value;

			$password = self::general_password_check($password);

			if (is_wp_error($password))
				return $password;



			return  $password;
		}

			/**
		 * @param $password
		 *
		 * @return WP_Error|string
		 */
		public static function general_password_check($password){

			if ( empty( $password ) ) {
				return new WP_Error( 'rest_user_invalid_password', __( 'Passwords cannot be empty.' , 'app-expert' ), array( 'status' => 400 ) );
			}

			if ( false !== strpos( $password, '\\' ) ) {
				return new WP_Error( 'rest_user_invalid_password', __( 'Passwords cannot contain the "\\" character.' , 'app-expert' ), array( 'status' => 400 ) );
			}

			if (strlen($password) < 6){
				return new WP_Error(
					'app_expert_password_short',
					__( 'Password is too short.' , 'app-expert' ),
					array( 'status' => 400 ) );
			}

			return $password;
		}


		/**
		 * @param mixed           $value   The password submitted in the request.
		 * @param WP_REST_Request $request Full details about the request.
		 * @param string          $param   The parameter name.
		 * @return WP_Error|string The sanitized password, if valid, otherwise an error.
		 */
		public function check_confirm_password($value, $request, $param)
		{
			$confirm_password = (string) $value;

			$password = $request->get_params()['password'];

			if (empty($confirm_password)) {
				return new WP_Error('rest_user_invalid_confirm_password', __('Confirm Password cannot be empty.', 'app-expert'), array('status' => 400));
			}

			if (false !== strpos($confirm_password, '\\')) {
				return new WP_Error('rest_user_invalid_confirm_password', __('Confirm Password cannot contain the "\\" character.', 'app-expert'), array('status' => 400));
			}

			if (!isset($password) || empty($password) || $password != $confirm_password) {
				return new WP_Error('rest_user_mismatch_password', __('Both Passwords Must Match', 'app-expert'), array('status' => 400));
			}


			return $confirm_password;
		}

		/**
		 * @param WP_REST_Request $request
		 *
		 * @return bool
		 */
		private function is_username_exists(WP_REST_Request $request)
		{

			$username = $request->get_param('username');

			return isset($username) && !empty($username);
		}
	}
}

$app_expert_register_controller = new App_Expert_Register_Controller();
$app_expert_register_controller->register_routes();
