<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Taxonomies_Controller')) {

	class App_Expert_Taxonomies_Controller extends WP_REST_Taxonomies_Controller {

		public function __construct() {
			$this->rest_base = '/taxonomies';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
		}


		public function register_routes() {

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base,
				array(
					array(
						'methods'             => WP_REST_Server::READABLE,
						'callback'            => array( $this, 'get_items' ),
						'args'                => $this->get_collection_params(),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);
		}

		public function get_items($request){

			$response =  parent::get_items($request);
			if (is_wp_error($response))
			 	return $response;

			$taxonomies = $response->get_data();

			
			if ( class_exists( 'WooCommerce' ) ) {
				try{
					$woocommerce_cat_tax = get_taxonomies(['name' => 'product_cat'], 'objects' );
					$woocommerce_cat_tax['product_cat']->rest_base = 'products/categories';
					$woocommerce_tag_tax = get_taxonomies(['name' => 'product_tag'], 'objects' );
					$woocommerce_tag_tax['product_tag']->rest_base = 'products/tags';
					$woocommerce_taxonomies = array_merge($woocommerce_cat_tax, $woocommerce_tag_tax);
					foreach ( $woocommerce_taxonomies as $tax_type => $value ) {	
						$tax               = $this->prepare_item_for_response( $value, $request );
						$tax               = $this->prepare_response_for_collection( $tax );
						$taxonomies[ $tax_type ] = $tax;
					}
				}catch(\Exception $e){
					error_log('An error when trying to get woocommerce taxonomies');
				}
			}

			$response->set_data($taxonomies);

			return App_Expert_Response::get_api_list_success_response(
				$response,
				'app_expert_taxonomy_types_retrieved',
				'retrieved all taxonomy Types'
			);

		}

	}

}

$app_expert_taxonomies_controller = new App_Expert_Taxonomies_Controller();
$app_expert_taxonomies_controller->register_routes();