<?php
if (!defined('ABSPATH')){
	exit();
}

if (!class_exists('App_Expert_Users_Controller')){

	class App_Expert_Users_Controller extends  WP_REST_Users_Controller {

		const USER_IMAGE_SMALL = 200;
		const USER_IMAGE_MEDIUM = 350;
		const USER_IMAGE_LARGE = 650;

		public function __construct() {

			parent::__construct();
			$this->rest_base = '/users';
			$this->namespace = APP_EXPERT_API_NAMESPACE;
		}

		public function register_routes() {

			register_rest_route(
				$this->namespace,
				'/' . $this->rest_base . '/me',
				array(
					array(
						'methods'  => WP_REST_Server::READABLE,
						'callback' => array( $this, 'get_current_item' ),
					),
					array(
						'methods'             => WP_REST_Server::EDITABLE,
						'callback'            => array( $this, 'update_current_item' ),
						'args'                => $this->get_endpoint_args_for_item_schema( WP_REST_Server::EDITABLE ),
					),
					'schema' => array( $this, 'get_public_item_schema' ),
				)
			);
		}

		public function get_current_item( $request ) {
			$user_id = App_Expert_Request::get_user_id($request);
			$user     = $this->get_user((int) $user_id);

			if ( is_wp_error( $user ) )
				return $user;


			$response = $this->prepare_item_for_response( $user, $request );
			$response = rest_ensure_response( $response );

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_retrieved_user',
				'Retrieved User Info'
			);
		}

		public function update_current_item( $request ) {
			$user_id = App_Expert_Request::get_user_id($request);

			if (isset($user_id) && !empty($user_id))
				$request['id'] = $user_id;


			$request = App_Expert_Request::remove_user_meta_data_update($request);

			$response = parent::update_item($request);

			if (is_wp_error($response))
				return$response;

			return App_Expert_Response::get_api_single_success_response(
				$response,
				'app_expert_updated_user',
				'Updated User Info'
			);

		}

		public function prepare_item_for_response($user , $request){
			return App_Expert_Response::prepare_user_data_for_response($user);
		}

		public function get_item_schema() {

			return array(
				'$schema'    => 'http://json-schema.org/draft-04/schema#',
				'title'      => 'user',
				'type'       => 'object',
				'properties' => array(
					'first_name'         => array(
						'description' => __( 'First name for the user.' ),
						'type'        => 'string',
						'context'     => array( 'edit' ),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
					),
					'last_name'          => array(
						'description' => __( 'Last name for the user.' ),
						'type'        => 'string',
						'context'     => array( 'edit' ),
						'arg_options' => array(
							'sanitize_callback' => 'sanitize_text_field',
						),
					),
					'email'              => array(
						'description' => __( 'The email address for the user.' ),
						'type'        => 'string',
						'format'      => 'email',
						'context'     => array( 'edit' ),
					),

				),
			);
		}

	}
}


$app_expert_users_controller = new App_Expert_Users_Controller();
$app_expert_users_controller->register_routes();